//
//  CWJourney.h
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import <Foundation/Foundation.h>
#import "CWLine.h"

@class CWPoint;

/*!
 * @abstract Target time for journey searches.
 */
typedef enum {
  CWJourneyDirectionDeparture = 0,  //! Match eparture time.
  CWJourneyDirectionArrival = 1     //! Match arrival time.
} CWJourneyDirection;

/*!
 * @abstract One suggestion for a searched journey.
 */
@interface CWJourney : NSObject <NSCoding> {
@private
  NSMutableArray* _routeLinks;
}

@property(nonatomic, readonly, retain) NSDate* departure;           //! Departure of first route link.
@property(nonatomic, readonly, retain) NSDate* arrival;             //! Arrival of first route link.
@property(nonatomic, readonly, retain) CWPoint* from;               //! First point to travel from. 
@property(nonatomic, readonly, retain) CWPoint* to;                 //! Last point to travel to.
@property(nonatomic, readonly, assign) CWLineType firstLineType;    //! Line type of the first route link.
@property(nonatomic, readonly, assign) NSUInteger numberOfChanges;  //! Number of changes, one route link is 0 changes.
@property(nonatomic, readonly, retain) NSArray* routeLinks;         //! An array of one or more route links.

/*!
 * @abstract Query if any of the route links has any kind of deviation.
 */
-(BOOL)isDeviated;

/*!
 * @abstract Fetch an array of journeys mathching a search.
 *
 * @discussion Requery swapping to and from for return journeys.
 */
+(NSArray*)journeysFrom:(CWPoint*)from to:(CWPoint*)to at:(NSDate*)time inDirection:(CWJourneyDirection)direction;

/*!
 * @abstract Fetch an array of journey with departure time before a previosly fetched journey.
 */
+(NSArray*)journeysBefore:(CWJourney*)journey;

/*!
 * @abstract Fetch an array of journey with departure time after a previosly fetched journey.
 */
+(NSArray*)journeysAfter:(CWJourney*)journey;

@end
