//
//  CWJourney.m
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "CWJourney.h"
#import "CWXMLTranslater.h"
#import "CWPoint.h"
#import "CWRouteLink.h"
#import "NSDate+CWExtentions.h"
#import "CWModel.h"
#import "CWTranslatedString.h"

@interface CWJourneyParser : NSObject <CWXMLTranslaterDelegate> {
@private
  NSURL* url;
  NSMutableArray* journeys;
  CWModel* sharedModel;
}

-(id)initWithFrom:(CWPoint*)from to:(CWPoint*)to at:(NSDate*)time inDirection:(CWJourneyDirection)direction;
-(id)initWithBeforeJourney:(CWJourney*)journey;
-(id)initWithAfterJourney:(CWJourney*)journey;

-(NSArray*)parseJourneys;

@end


@implementation CWJourney

-(CWPoint*)from;
{
  return ((CWRouteLink*)[self.routeLinks objectAtIndex:0]).from;
}

-(NSDate*)departure;
{
  return ((CWRouteLink*)[self.routeLinks objectAtIndex:0]).departure;
}

-(CWPoint*)to;
{
  return ((CWRouteLink*)[self.routeLinks lastObject]).to;
}

-(NSDate*)arrival;
{
  return ((CWRouteLink*)[self.routeLinks lastObject]).arrival;
}

-(NSUInteger)numberOfChanges;
{
  return [self.routeLinks count] - 1;
}

-(CWLineType)firstLineType;
{
  return ((CWRouteLink*)[self.routeLinks objectAtIndex:0]).line.type;
}

-(BOOL)isDeviated;
{
	for (CWRouteLink* link in self.routeLinks) {
  	if ([link isDeviated]) {
    	return YES;
    }
  }
  return NO;
}

@synthesize routeLinks = _routeLinks;

+(NSArray*)journeysFrom:(CWPoint*)from to:(CWPoint*)to at:(NSDate*)time inDirection:(CWJourneyDirection)direction;
{
  CWJourneyParser* parser = [[CWJourneyParser alloc] initWithFrom:from to:to at:time inDirection:direction];
  NSArray* journeys = [parser parseJourneys];
  NSLog(@"%@", [journeys description]);
  [parser release];
  return journeys;
}

+(NSArray*)journeysBefore:(CWJourney*)journey;
{
  CWJourneyParser* parser = [[CWJourneyParser alloc] initWithBeforeJourney:journey];
  NSArray* journeys = [parser parseJourneys];
  NSLog(@"%@", [journeys description]);
  [parser release];
  return journeys;
}

+(NSArray*)journeysAfter:(CWJourney*)journey;
{
  CWJourneyParser* parser = [[CWJourneyParser alloc] initWithAfterJourney:journey];
  NSArray* journeys = [parser parseJourneys];
  NSLog(@"%@", [journeys description]);
  [parser release];
  return journeys;  
}

-(id)initWithCoder:(NSCoder *)aDecoder;
{
  self = [self init];
  if (self) {
    _routeLinks = [[aDecoder decodeObjectForKey:@"routeLinks"] retain];
  }
  return self;
}

-(void)encodeWithCoder:(NSCoder *)aCoder;
{
  [aCoder encodeObject:self.routeLinks forKey:@"routeLinks"];
}

-(void)dealloc;
{
  [_routeLinks release];
  [super dealloc];
}

-(BOOL)isEqual:(id)anObject;
{
  if (self == anObject) {
    return YES;
  } else if ([anObject isKindOfClass:[CWJourney class]]) {
    CWJourney* otherJourney = anObject;
    return [self.routeLinks isEqualToArray:otherJourney.routeLinks];
  }
  return NO;
}

-(NSString*)description;
{
  return [NSString stringWithFormat:@"<CWRouteLink: %@ at %@, %@ at %@, %@", self.from, self.departure, self.to, self.arrival, self.routeLinks];
}

@end

@implementation CWJourneyParser


+(NSURL*)queryURLForFrom:(CWPoint*)from to:(CWPoint*)to at:(NSDate*)time inDirection:(CWJourneyDirection)direction;
{
  static NSString* queryFormat = @"resultspage.asp?cmdaction=search&selpointfr=%@&selpointto=%@&inpDate=%@&inpTime=%@&selDirection=%d";
  NSString* query = [NSString stringWithFormat:queryFormat, from.queryString, to.queryString, [time compactISODate], [time compactISOTime], direction];
  query = [[[[NSBundle mainBundle] infoDictionary] objectForKey:@"ServerURL"] stringByAppendingString:query];
  NSURL* url = [NSURL URLWithString:query];
  NSLog(@"%@", url);
  return url;
}

+(NSURL*)queryURLForBeforeJourney:(CWJourney*)journey;
{
  static NSString* queryFormat = @"resultspage.asp?FirstStart=%@%%20%@&selpointfr=%@&selpointto=%@&cmdaction=previous";
  CWPoint* from = journey.from;
  CWPoint* to = journey.to;
  NSDate* time = journey.departure;
  NSString* query = [NSString stringWithFormat:queryFormat, [time ISODate], [time ISOTime], from.queryString, to.queryString];
  query = [[[[NSBundle mainBundle] infoDictionary] objectForKey:@"ServerURL"] stringByAppendingString:query];
  NSURL* url = [NSURL URLWithString:query];
  NSLog(@"%@", url);
  return url;
}

+(NSURL*)queryURLForAfterJourney:(CWJourney*)journey;
{
  static NSString* queryFormat = @"resultspage.asp?LastStart=%@%%20%@&selpointfr=%@&selpointto=%@&cmdaction=next";
  CWPoint* from = journey.from;
  CWPoint* to = journey.to;
  NSDate* time = journey.departure;
  NSString* query = [NSString stringWithFormat:queryFormat, [time ISODate], [time ISOTime], from.queryString, to.queryString];
  query = [[[[NSBundle mainBundle] infoDictionary] objectForKey:@"ServerURL"] stringByAppendingString:query];
  NSURL* url = [NSURL URLWithString:query];
  NSLog(@"%@", url);
  return url;
}

-(id)initWithFrom:(CWPoint*)from to:(CWPoint*)to at:(NSDate*)time inDirection:(CWJourneyDirection)direction;
{
  self = [super init];
  if (self) {
    url = [[CWJourneyParser queryURLForFrom:from to:to at:time inDirection:direction] retain];
    journeys = [NSMutableArray new];
    sharedModel = [CWModel sharedModel];
  }
  return self;
}

-(id)initWithBeforeJourney:(CWJourney*)journey;
{
  self = [super init];
  if (self) {
    url = [[CWJourneyParser queryURLForBeforeJourney:journey] retain];
    journeys = [NSMutableArray new];
    sharedModel = [CWModel sharedModel];
  }
  return self;
}

-(id)initWithAfterJourney:(CWJourney*)journey;
{
  self = [super init];
  if (self) {
    url = [[CWJourneyParser queryURLForAfterJourney:journey] retain];
    journeys = [NSMutableArray new];
    sharedModel = [CWModel sharedModel];
  }
  return self;  
}

-(void)dealloc;
{
  [url release];
  [journeys release];
  [super dealloc];
}

-(NSArray*)parseJourneys;
{
  [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
  NSDate* delayDate = [NSDate dateWithTimeIntervalSinceNow:1.0];
  NSDictionary* translation = [NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"CWJourneyParser" ofType:@"plist"]];
	CWXMLTranslater* translater = [[CWXMLTranslater alloc] initWithTranslationPropertyList:translation delegate:self];
	NSError* error = nil;
  if (![translater translateContentsOfURL:url delegate:translater.delegate error:&error]) {
    NSLog(@"Error: %@", [error description]);
    NSLog(@"URL: %@", [url description]);
    //NSLog(@"XML: %@", [NSString stringWithContentsOfURL:url encoding:NSUTF8StringEncoding error:NULL]);
  	UIAlertView* alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"FailedNetwork", nil) message:NSLocalizedString(@"FailedNetworkMessage", nil) delegate:nil cancelButtonTitle:NSLocalizedString(@"OK", nil) otherButtonTitles:nil];
    [alert show];
    [alert release];
    journeys = nil;
  } else {
  	[NSThread sleepUntilDate:delayDate];
  }
  [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
	return [[journeys retain] autorelease];
}

-(void)xmlTranslater:(CWXMLTranslater*)translater didTranslateObject:(id)anObject forKey:(NSString*)key;
{
	[journeys addObject:anObject];
}

-(id)xmlTranslater:(CWXMLTranslater*)translater willTranslateValueOfClass:(Class)aClass withString:(NSString*)aString forKey:(NSString*)key;
{
	if (aClass == [NSDate class]) {
  	return [NSDate dateWithISODateString:aString];
  } else if ([key isEqualToString:@"name"]) {
  	return [aString capitalizedString];
  } else if (sharedModel.translateTexts && ([key isEqualToString:@"header"] || [key isEqualToString:@"shortText"])) {
		return CWTranslatedString(aString, @"sv");    
  }
  return nil;
}

@end
