//
//  CWJourneyDetailController.m
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "CWJourneyDetailController.h"
#import "NSDate+CWExtentions.h"
#import "CWJourneyDetailCell.h"

@implementation CWJourneyDetailController

-(id)initWithJourney:(CWJourney*)aJourney;
{
	self = [self initWithStyle:UITableViewStylePlain];
  if (self) {
  	journey = [aJourney retain];
    self.title = [NSString stringWithFormat:NSLocalizedString(@"FromToTime", nil), [journey.departure localizedShortTimeString], [journey.arrival localizedShortTimeString]];
  }
  return self;
}

-(void)viewDidLoad;
{
	self.tableView.allowsSelection = NO;
}

-(void)dealloc;
{
 	[journey release];
  [super dealloc];
}


#pragma mark Table view methods

// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section;
{
	return [journey.routeLinks count];
}


-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath;
{
	CWRouteLink* routeLink = [journey.routeLinks objectAtIndex:indexPath.row];
  NSString* deviationsString = routeLink.deviationsAsString;
  if (deviationsString) {
    CGSize size = CGSizeMake(self.tableView.bounds.size.width - 22, 400);
    size = [deviationsString sizeWithFont:[UIFont systemFontOfSize:14.f] constrainedToSize:size lineBreakMode:UILineBreakModeWordWrap];
 		return 110.f + size.height;
  } else {
    return 110.f;
  }
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath;
{
  static NSString *CellIdentifier = @"Cell";
  
  CWJourneyDetailCell *cell = (CWJourneyDetailCell*)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if (cell == nil) {
    cell = [CWJourneyDetailCell journeyDetailCell];
  }
	CWRouteLink* routeLink = [journey.routeLinks objectAtIndex:indexPath.row];
  
	[cell setRouteLink:routeLink];
  
  return cell;  
}

@end

